@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2013-2017 Gauvain Pocentek <gauvain@pocentek.net>
+# Copyright (C) 2013-2019 Gauvain Pocentek, 2019-2023 python-gitlab team
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -43,11 +43,25 @@ def __getattr__(name: str) -> Any:
     if name in gitlab.const._DEPRECATED:
         _utils.warn(
             message=(
-                f"\nDirect access to 'gitlab.{name}' is deprecated and will be "
-                f"removed in a future major python-gitlab release. Please "
-                f"use 'gitlab.const.{name}' instead."
+                f"\nDirect access to constants as 'gitlab.{name}' is deprecated and "
+                f"will be removed in a future major python-gitlab release. Please "
+                f"see the usage of constants in the 'gitlab.const' module instead."
             ),
             category=DeprecationWarning,
         )
         return getattr(gitlab.const, name)
     raise AttributeError(f"module {__name__} has no attribute {name}")
+
+
+__all__ = [
+    "__author__",
+    "__copyright__",
+    "__email__",
+    "__license__",
+    "__title__",
+    "__version__",
+    "Gitlab",
+    "GitlabList",
+]
+__all__.extend(gitlab.const._DEPRECATED)
+__all__.extend(gitlab.exceptions.__all__)
