@@ -3,18 +3,16 @@
 import os
 import re
 import time
-from typing import Any, cast, Dict, List, Optional, Tuple, TYPE_CHECKING, Union
+from typing import Any, cast, Dict, List, Optional, Tuple, Type, TYPE_CHECKING, Union
 from urllib import parse
 
 import requests
-import requests.utils
-from requests_toolbelt.multipart.encoder import MultipartEncoder  # type: ignore
 
 import gitlab
 import gitlab.config
 import gitlab.const
 import gitlab.exceptions
-from gitlab import utils
+from gitlab import _backends, utils
 
 REDIRECT_MSG = (
     "python-gitlab detected a {status_code} ({reason!r}) redirection. You must update "
@@ -22,7 +20,6 @@ REDIRECT_MSG = (
     "{source!r} to {target!r}"
 )
 
-RETRYABLE_TRANSIENT_ERROR_CODES = [500, 502, 503, 504] + list(range(520, 531))
 
 # https://docs.gitlab.com/ee/api/#offset-based-pagination
 _PAGINATION_URL = (
@@ -32,6 +29,7 @@ _PAGINATION_URL = (
 
 
 class Gitlab:
+
     """Represents a GitLab server connection.
 
     Args:
@@ -53,6 +51,10 @@ class Gitlab:
             or 52x responses. Defaults to False.
         keep_base_url: keep user-provided base URL for pagination if it
             differs from response headers
+
+    Keyword Args:
+        requests.Session session: HTTP Requests Session
+        RequestsBackend backend: Backend that will be used to make http requests
     """
 
     def __init__(
@@ -66,15 +68,14 @@ class Gitlab:
         http_password: Optional[str] = None,
         timeout: Optional[float] = None,
         api_version: str = "4",
-        session: Optional[requests.Session] = None,
         per_page: Optional[int] = None,
         pagination: Optional[str] = None,
         order_by: Optional[str] = None,
         user_agent: str = gitlab.const.USER_AGENT,
         retry_transient_errors: bool = False,
         keep_base_url: bool = False,
+        **kwargs: Any,
     ) -> None:
-
         self._api_version = str(api_version)
         self._server_version: Optional[str] = None
         self._server_revision: Optional[str] = None
@@ -98,7 +99,11 @@ class Gitlab:
         self._set_auth_info()
 
         #: Create a session object for requests
-        self.session = session or requests.Session()
+        _backend: Type[_backends.DefaultBackend] = kwargs.pop(
+            "backend", _backends.DefaultBackend
+        )
+        self._backend = _backend(**kwargs)
+        self.session = self._backend.client
 
         self.per_page = per_page
         self.pagination = pagination
@@ -116,6 +121,10 @@ class Gitlab:
 
         self.broadcastmessages = objects.BroadcastMessageManager(self)
         """See :class:`~gitlab.v4.objects.BroadcastMessageManager`"""
+        self.bulk_imports = objects.BulkImportManager(self)
+        """See :class:`~gitlab.v4.objects.BulkImportManager`"""
+        self.bulk_import_entities = objects.BulkImportAllEntityManager(self)
+        """See :class:`~gitlab.v4.objects.BulkImportAllEntityManager`"""
         self.ci_lint = objects.CiLintManager(self)
         """See :class:`~gitlab.v4.objects.CiLintManager`"""
         self.deploykeys = objects.DeployKeyManager(self)
@@ -188,6 +197,8 @@ class Gitlab:
         """See :class:`~gitlab.v4.objects.PersonalAccessTokenManager`"""
         self.topics = objects.TopicManager(self)
         """See :class:`~gitlab.v4.objects.TopicManager`"""
+        self.statistics = objects.ApplicationStatisticsManager(self)
+        """See :class:`~gitlab.v4.objects.ApplicationStatisticsManager`"""
 
     def __enter__(self) -> "Gitlab":
         return self
@@ -230,7 +241,10 @@ class Gitlab:
 
     @classmethod
     def from_config(
-        cls, gitlab_id: Optional[str] = None, config_files: Optional[List[str]] = None
+        cls,
+        gitlab_id: Optional[str] = None,
+        config_files: Optional[List[str]] = None,
+        **kwargs: Any,
     ) -> "Gitlab":
         """Create a Gitlab connection from configuration files.
 
@@ -238,6 +252,9 @@ class Gitlab:
             gitlab_id: ID of the configuration section.
             config_files list[str]: List of paths to configuration files.
 
+        kwargs:
+            session requests.Session: Custom requests Session
+
         Returns:
             A Gitlab connection.
 
@@ -262,6 +279,7 @@ class Gitlab:
             order_by=config.order_by,
             user_agent=config.user_agent,
             retry_transient_errors=config.retry_transient_errors,
+            **kwargs,
         )
 
     @classmethod
@@ -515,7 +533,7 @@ class Gitlab:
             self.headers.pop("PRIVATE-TOKEN", None)
             self.headers["JOB-TOKEN"] = self.job_token
 
-        if self.http_username:
+        if self.http_username and self.http_password:
             self._http_auth = requests.auth.HTTPBasicAuth(
                 self.http_username, self.http_password
             )
@@ -621,38 +639,6 @@ class Gitlab:
                 )
             )
 
-    @staticmethod
-    def _prepare_send_data(
-        files: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
-        raw: bool = False,
-    ) -> Tuple[
-        Optional[Union[Dict[str, Any], bytes]],
-        Optional[Union[Dict[str, Any], MultipartEncoder]],
-        str,
-    ]:
-        if files:
-            if post_data is None:
-                post_data = {}
-            else:
-                # booleans does not exists for data (neither for MultipartEncoder):
-                # cast to string int to avoid: 'bool' object has no attribute 'encode'
-                if TYPE_CHECKING:
-                    assert isinstance(post_data, dict)
-                for k, v in post_data.items():
-                    if isinstance(v, bool):
-                        post_data[k] = str(int(v))
-            post_data["file"] = files.get("file")
-            post_data["avatar"] = files.get("avatar")
-
-            data = MultipartEncoder(post_data)
-            return (None, data, data.content_type)
-
-        if raw and post_data:
-            return (None, post_data, "application/octet-stream")
-
-        return (post_data, None, "application/json")
-
     def http_request(
         self,
         verb: str,
@@ -730,17 +716,17 @@ class Gitlab:
             retry_transient_errors = self.retry_transient_errors
 
         # We need to deal with json vs. data when uploading files
-        json, data, content_type = self._prepare_send_data(files, post_data, raw)
-        opts["headers"]["Content-type"] = content_type
+        send_data = self._backend.prepare_send_data(files, post_data, raw)
+        opts["headers"]["Content-type"] = send_data.content_type
 
         cur_retries = 0
         while True:
             try:
-                result = self.session.request(
+                result = self._backend.http_request(
                     method=verb,
                     url=url,
-                    json=json,
-                    data=data,
+                    json=send_data.json,
+                    data=send_data.data,
                     params=params,
                     timeout=timeout,
                     verify=verify,
@@ -758,15 +744,25 @@ class Gitlab:
 
                 raise
 
-            self._check_redirects(result)
+            self._check_redirects(result.response)
 
             if 200 <= result.status_code < 300:
-                return result
+                return result.response
 
-            if (429 == result.status_code and obey_rate_limit) or (
-                result.status_code in RETRYABLE_TRANSIENT_ERROR_CODES
-                and retry_transient_errors
-            ):
+            def should_retry() -> bool:
+                if result.status_code == 429 and obey_rate_limit:
+                    return True
+
+                if not retry_transient_errors:
+                    return False
+                if result.status_code in gitlab.const.RETRYABLE_TRANSIENT_ERROR_CODES:
+                    return True
+                if result.status_code == 409 and "Resource lock" in result.reason:
+                    return True
+
+                return False
+
+            if should_retry():
                 # Response headers documentation:
                 # https://docs.gitlab.com/ee/user/admin_area/settings/user_and_ip_rate_limits.html#response-headers
                 if max_retries == -1 or cur_retries < max_retries:
@@ -832,12 +828,9 @@ class Gitlab:
         result = self.http_request(
             "get", path, query_data=query_data, streamed=streamed, **kwargs
         )
+        content_type = utils.get_content_type(result.headers.get("Content-Type"))
 
-        if (
-            result.headers["Content-Type"] == "application/json"
-            and not streamed
-            and not raw
-        ):
+        if content_type == "application/json" and not streamed and not raw:
             try:
                 json_result = result.json()
                 if TYPE_CHECKING:
@@ -931,7 +924,20 @@ class Gitlab:
 
         page = kwargs.get("page")
 
-        if iterator:
+        if iterator and page is not None:
+            arg_used_message = f"iterator={iterator}"
+            if as_list is not None:
+                arg_used_message = f"as_list={as_list}"
+            utils.warn(
+                message=(
+                    f"`{arg_used_message}` and `page={page}` were both specified. "
+                    f"`{arg_used_message}` will be ignored and a `list` will be "
+                    f"returned."
+                ),
+                category=UserWarning,
+            )
+
+        if iterator and page is None:
             # Generator requested
             return GitlabList(self, url, query_data, **kwargs)
 
@@ -1021,8 +1027,10 @@ class Gitlab:
             raw=raw,
             **kwargs,
         )
+        content_type = utils.get_content_type(result.headers.get("Content-Type"))
+
         try:
-            if result.headers.get("Content-Type", None) == "application/json":
+            if content_type == "application/json":
                 json_result = result.json()
                 if TYPE_CHECKING:
                     assert isinstance(json_result, dict)
@@ -1083,6 +1091,54 @@ class Gitlab:
                 error_message="Failed to parse the server message"
             ) from e
 
+    def http_patch(
+        self,
+        path: str,
+        *,
+        query_data: Optional[Dict[str, Any]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
+        raw: bool = False,
+        **kwargs: Any,
+    ) -> Union[Dict[str, Any], requests.Response]:
+        """Make a PATCH request to the Gitlab server.
+
+        Args:
+            path: Path or full URL to query ('/projects' or
+                        'http://whatever/v4/api/projecs')
+            query_data: Data to send as query parameters
+            post_data: Data to send in the body (will be converted to
+                              json by default)
+            raw: If True, do not convert post_data to json
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Returns:
+            The parsed json returned by the server.
+
+        Raises:
+            GitlabHttpError: When the return code is not 2xx
+            GitlabParsingError: If the json data could not be parsed
+        """
+        query_data = query_data or {}
+        post_data = post_data or {}
+
+        result = self.http_request(
+            "patch",
+            path,
+            query_data=query_data,
+            post_data=post_data,
+            raw=raw,
+            **kwargs,
+        )
+        try:
+            json_result = result.json()
+            if TYPE_CHECKING:
+                assert isinstance(json_result, dict)
+            return json_result
+        except Exception as e:
+            raise gitlab.exceptions.GitlabParsingError(
+                error_message="Failed to parse the server message"
+            ) from e
+
     def http_delete(self, path: str, **kwargs: Any) -> requests.Response:
         """Make a DELETE request to the Gitlab server.
 
