@@ -1,21 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-#
-# Copyright (C) 2013-2017 Gauvain Pocentek <gauvain@pocentek.net>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Lesser General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 import argparse
 import operator
 import sys
@@ -210,8 +192,16 @@ class GitlabCLI:
         return result
 
 
+# https://github.com/python/typeshed/issues/7539#issuecomment-1076581049
+if TYPE_CHECKING:
+    _SubparserType = argparse._SubParsersAction[argparse.ArgumentParser]
+else:
+    _SubparserType = Any
+
+
 def _populate_sub_parser_by_class(
-    cls: Type[gitlab.base.RESTObject], sub_parser: argparse._SubParsersAction
+    cls: Type[gitlab.base.RESTObject],
+    sub_parser: _SubparserType,
 ) -> None:
     mgr_cls_name = f"{cls.__name__}Manager"
     mgr_cls = getattr(gitlab.v4.objects, mgr_cls_name)
@@ -301,9 +291,11 @@ def _populate_sub_parser_by_class(
         for action_name in cli.custom_actions[name]:
             # NOTE(jlvillal): If we put a function for the `default` value of
             # the `get` it will always get called, which will break things.
-            sub_parser_action = action_parsers.get(action_name)
-            if sub_parser_action is None:
+            action_parser = action_parsers.get(action_name)
+            if action_parser is None:
                 sub_parser_action = sub_parser.add_parser(action_name)
+            else:
+                sub_parser_action = action_parser
             # Get the attributes for URL/path construction
             if mgr_cls._from_parent_attrs:
                 for x in mgr_cls._from_parent_attrs:
@@ -335,9 +327,11 @@ def _populate_sub_parser_by_class(
         for action_name in cli.custom_actions[name]:
             # NOTE(jlvillal): If we put a function for the `default` value of
             # the `get` it will always get called, which will break things.
-            sub_parser_action = action_parsers.get(action_name)
-            if sub_parser_action is None:
+            action_parser = action_parsers.get(action_name)
+            if action_parser is None:
                 sub_parser_action = sub_parser.add_parser(action_name)
+            else:
+                sub_parser_action = action_parser
             if mgr_cls._from_parent_attrs:
                 for x in mgr_cls._from_parent_attrs:
                     sub_parser_action.add_argument(
@@ -393,9 +387,9 @@ def extend_parser(parser: argparse.ArgumentParser) -> argparse.ArgumentParser:
 
 
 def get_dict(
-    obj: Union[str, gitlab.base.RESTObject], fields: List[str]
+    obj: Union[str, Dict[str, Any], gitlab.base.RESTObject], fields: List[str]
 ) -> Union[str, Dict[str, Any]]:
-    if isinstance(obj, str):
+    if not isinstance(obj, gitlab.base.RESTObject):
         return obj
 
     if fields:
@@ -412,7 +406,7 @@ class JSONPrinter:
 
     @staticmethod
     def display_list(
-        data: List[Union[str, gitlab.base.RESTObject]],
+        data: List[Union[str, Dict[str, Any], gitlab.base.RESTObject]],
         fields: List[str],
         **_kwargs: Any,
     ) -> None:
@@ -437,7 +431,7 @@ class YAMLPrinter:
 
     @staticmethod
     def display_list(
-        data: List[Union[str, gitlab.base.RESTObject]],
+        data: List[Union[str, Dict[str, Any], gitlab.base.RESTObject]],
         fields: List[str],
         **_kwargs: Any,
     ) -> None:
@@ -488,23 +482,35 @@ class LegacyPrinter:
             if obj._id_attr:
                 attrs.pop(obj._id_attr)
             display_dict(attrs, padding)
+            return
 
-        else:
-            if TYPE_CHECKING:
-                assert isinstance(obj, gitlab.base.RESTObject)
-            if obj._id_attr:
-                id = getattr(obj, obj._id_attr)
-                print(f"{obj._id_attr.replace('_', '-')}: {id}")
-            if obj._repr_attr:
-                value = getattr(obj, obj._repr_attr, "None")
-                value = value.replace("\r", "").replace("\n", " ")
-                # If the attribute is a note (ProjectCommitComment) then we do
-                # some modifications to fit everything on one line
-                line = f"{obj._repr_attr}: {value}"
-                # ellipsize long lines (comments)
-                if len(line) > 79:
-                    line = f"{line[:76]}..."
-                print(line)
+        lines = []
+
+        if TYPE_CHECKING:
+            assert isinstance(obj, gitlab.base.RESTObject)
+
+        if obj._id_attr:
+            id = getattr(obj, obj._id_attr)
+            lines.append(f"{obj._id_attr.replace('_', '-')}: {id}")
+        if obj._repr_attr:
+            value = getattr(obj, obj._repr_attr, "None") or "None"
+            value = value.replace("\r", "").replace("\n", " ")
+            # If the attribute is a note (ProjectCommitComment) then we do
+            # some modifications to fit everything on one line
+            line = f"{obj._repr_attr}: {value}"
+            # ellipsize long lines (comments)
+            if len(line) > 79:
+                line = f"{line[:76]}..."
+            lines.append(line)
+
+        if lines:
+            print("\n".join(lines))
+            return
+
+        print(
+            f"No default fields to show for {obj!r}. "
+            f"Please use  '--verbose' or the JSON/YAML formatters."
+        )
 
     def display_list(
         self,
@@ -553,6 +559,8 @@ def run(
         printer.display(data, verbose=True, obj=data)
     elif isinstance(data, list):
         printer.display_list(data, fields, verbose=verbose)
+    elif isinstance(data, gitlab.base.RESTObjectList):
+        printer.display_list(list(data), fields, verbose=verbose)
     elif isinstance(data, gitlab.base.RESTObject):
         printer.display(get_dict(data, fields), verbose=verbose, obj=data)
     elif isinstance(data, str):
