@@ -5,9 +5,17 @@ import requests
 import gitlab
 from gitlab import cli
 from gitlab import exceptions as exc
-from gitlab import types
+from gitlab import types, utils
 from gitlab.base import RESTManager, RESTObject
-from gitlab.mixins import CRUDMixin, ListMixin, ObjectDeleteMixin, SaveMixin
+from gitlab.mixins import (
+    CreateMixin,
+    CRUDMixin,
+    DeleteMixin,
+    ListMixin,
+    NoUpdateMixin,
+    ObjectDeleteMixin,
+    SaveMixin,
+)
 from gitlab.types import RequiredOptional
 
 from .access_requests import GroupAccessRequestManager  # noqa: F401
@@ -15,13 +23,16 @@ from .audit_events import GroupAuditEventManager  # noqa: F401
 from .badges import GroupBadgeManager  # noqa: F401
 from .boards import GroupBoardManager  # noqa: F401
 from .clusters import GroupClusterManager  # noqa: F401
+from .container_registry import GroupRegistryRepositoryManager  # noqa: F401
 from .custom_attributes import GroupCustomAttributeManager  # noqa: F401
 from .deploy_tokens import GroupDeployTokenManager  # noqa: F401
 from .epics import GroupEpicManager  # noqa: F401
 from .export_import import GroupExportManager, GroupImportManager  # noqa: F401
 from .group_access_tokens import GroupAccessTokenManager  # noqa: F401
 from .hooks import GroupHookManager  # noqa: F401
+from .invitations import GroupInvitationManager  # noqa: F401
 from .issues import GroupIssueManager  # noqa: F401
+from .iterations import GroupIterationManager  # noqa: F401
 from .labels import GroupLabelManager  # noqa: F401
 from .members import (  # noqa: F401
     GroupBillableMemberManager,
@@ -32,7 +43,8 @@ from .merge_requests import GroupMergeRequestManager  # noqa: F401
 from .milestones import GroupMilestoneManager  # noqa: F401
 from .notification_settings import GroupNotificationSettingsManager  # noqa: F401
 from .packages import GroupPackageManager  # noqa: F401
-from .projects import GroupProjectManager  # noqa: F401
+from .projects import GroupProjectManager, SharedProjectManager  # noqa: F401
+from .push_rules import GroupPushRulesManager
 from .runners import GroupRunnerManager  # noqa: F401
 from .statistics import GroupIssuesStatisticsManager  # noqa: F401
 from .variables import GroupVariableManager  # noqa: F401
@@ -43,8 +55,12 @@ __all__ = [
     "GroupManager",
     "GroupDescendantGroup",
     "GroupDescendantGroupManager",
+    "GroupLDAPGroupLink",
+    "GroupLDAPGroupLinkManager",
     "GroupSubgroup",
     "GroupSubgroupManager",
+    "GroupSAMLGroupLink",
+    "GroupSAMLGroupLinkManager",
 ]
 
 
@@ -65,9 +81,12 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     exports: GroupExportManager
     hooks: GroupHookManager
     imports: GroupImportManager
+    invitations: GroupInvitationManager
     issues: GroupIssueManager
     issues_statistics: GroupIssuesStatisticsManager
+    iterations: GroupIterationManager
     labels: GroupLabelManager
+    ldap_group_links: "GroupLDAPGroupLinkManager"
     members: GroupMemberManager
     members_all: GroupMemberAllManager
     mergerequests: GroupMergeRequestManager
@@ -75,10 +94,14 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
     notificationsettings: GroupNotificationSettingsManager
     packages: GroupPackageManager
     projects: GroupProjectManager
+    shared_projects: SharedProjectManager
+    pushrules: GroupPushRulesManager
+    registry_repositories: GroupRegistryRepositoryManager
     runners: GroupRunnerManager
     subgroups: "GroupSubgroupManager"
     variables: GroupVariableManager
     wikis: GroupWikiManager
+    saml_group_links: "GroupSAMLGroupLinkManager"
 
     @cli.register_custom_action("Group", ("project_id",))
     @exc.on_http_error(exc.GitlabTransferProjectError)
@@ -159,6 +182,13 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server cannot perform the request
         """
+        utils.warn(
+            message=(
+                "The add_ldap_group_link() method is deprecated and will be removed "
+                "in a future version. Use ldap_group_links.create() instead."
+            ),
+            category=DeprecationWarning,
+        )
         path = f"/groups/{self.encoded_id}/ldap_group_links"
         data = {"cn": cn, "group_access": group_access, "provider": provider}
         self.manager.gitlab.http_post(path, post_data=data, **kwargs)
@@ -179,6 +209,13 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
             GitlabAuthenticationError: If authentication is not correct
             GitlabDeleteError: If the server cannot perform the request
         """
+        utils.warn(
+            message=(
+                "The delete_ldap_group_link() method is deprecated and will be "
+                "removed in a future version. Use ldap_group_links.delete() instead."
+            ),
+            category=DeprecationWarning,
+        )
         path = f"/groups/{self.encoded_id}/ldap_group_links"
         if provider is not None:
             path += f"/{provider}"
@@ -250,6 +287,21 @@ class Group(SaveMixin, ObjectDeleteMixin, RESTObject):
         path = f"/groups/{self.encoded_id}/share/{group_id}"
         self.manager.gitlab.http_delete(path, **kwargs)
 
+    @cli.register_custom_action("Group")
+    @exc.on_http_error(exc.GitlabRestoreError)
+    def restore(self, **kwargs: Any) -> None:
+        """Restore a  group marked for deletion..
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabRestoreError: If the server failed to perform the request
+        """
+        path = f"/groups/{self.encoded_id}/restore"
+        self.manager.gitlab.http_post(path, **kwargs)
+
 
 class GroupManager(CRUDMixin, RESTManager):
     _path = "/groups"
@@ -326,7 +378,7 @@ class GroupManager(CRUDMixin, RESTManager):
         file: BinaryIO,
         path: str,
         name: str,
-        parent_id: Optional[str] = None,
+        parent_id: Optional[Union[int, str]] = None,
         **kwargs: Any,
     ) -> Union[Dict[str, Any], requests.Response]:
         """Import a group from an archive file.
@@ -347,7 +399,7 @@ class GroupManager(CRUDMixin, RESTManager):
             A representation of the import status.
         """
         files = {"file": ("file.tar.gz", file, "application/octet-stream")}
-        data = {"path": path, "name": name}
+        data: Dict[str, Any] = {"path": path, "name": name}
         if parent_id is not None:
             data["parent_id"] = parent_id
 
@@ -390,3 +442,61 @@ class GroupDescendantGroupManager(GroupSubgroupManager):
 
     _path = "/groups/{group_id}/descendant_groups"
     _obj_cls: Type[GroupDescendantGroup] = GroupDescendantGroup
+
+
+class GroupLDAPGroupLink(RESTObject):
+    _repr_attr = "provider"
+
+    def _get_link_attrs(self) -> Dict[str, str]:
+        # https://docs.gitlab.com/ee/api/groups.html#add-ldap-group-link-with-cn-or-filter
+        # https://docs.gitlab.com/ee/api/groups.html#delete-ldap-group-link-with-cn-or-filter
+        # We can tell what attribute to use based on the data returned
+        data = {"provider": self.provider}
+        if self.cn:
+            data["cn"] = self.cn
+        else:
+            data["filter"] = self.filter
+
+        return data
+
+    def delete(self, **kwargs: Any) -> None:
+        """Delete the LDAP group link from the server.
+
+        Args:
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabDeleteError: If the server cannot perform the request
+        """
+        if TYPE_CHECKING:
+            assert isinstance(self.manager, DeleteMixin)
+        self.manager.delete(
+            self.encoded_id, query_data=self._get_link_attrs(), **kwargs
+        )
+
+
+class GroupLDAPGroupLinkManager(ListMixin, CreateMixin, DeleteMixin, RESTManager):
+    _path = "/groups/{group_id}/ldap_group_links"
+    _obj_cls: Type[GroupLDAPGroupLink] = GroupLDAPGroupLink
+    _from_parent_attrs = {"group_id": "id"}
+    _create_attrs = RequiredOptional(
+        required=("provider", "group_access"), exclusive=("cn", "filter")
+    )
+
+
+class GroupSAMLGroupLink(ObjectDeleteMixin, RESTObject):
+    _id_attr = "name"
+    _repr_attr = "name"
+
+
+class GroupSAMLGroupLinkManager(NoUpdateMixin, RESTManager):
+    _path = "/groups/{group_id}/saml_group_links"
+    _obj_cls: Type[GroupSAMLGroupLink] = GroupSAMLGroupLink
+    _from_parent_attrs = {"group_id": "id"}
+    _create_attrs = RequiredOptional(required=("saml_group_name", "access_level"))
+
+    def get(
+        self, id: Union[str, int], lazy: bool = False, **kwargs: Any
+    ) -> GroupSAMLGroupLink:
+        return cast(GroupSAMLGroupLink, super().get(id=id, lazy=lazy, **kwargs))
