@@ -1,9 +1,9 @@
-from typing import Any, cast, Dict, Tuple, TYPE_CHECKING, Union
+from typing import Any, cast, Dict, Optional, Tuple, TYPE_CHECKING, Union
 
 from gitlab import cli
 from gitlab import exceptions as exc
 from gitlab import types
-from gitlab.base import RequiredOptional, RESTManager, RESTObject
+from gitlab.base import RESTManager, RESTObject
 from gitlab.mixins import (
     CreateMixin,
     CRUDMixin,
@@ -18,13 +18,16 @@ from gitlab.mixins import (
     TodoMixin,
     UserAgentDetailMixin,
 )
+from gitlab.types import RequiredOptional
 
 from .award_emojis import ProjectIssueAwardEmojiManager  # noqa: F401
 from .discussions import ProjectIssueDiscussionManager  # noqa: F401
 from .events import (  # noqa: F401
+    ProjectIssueResourceIterationEventManager,
     ProjectIssueResourceLabelEventManager,
     ProjectIssueResourceMilestoneEventManager,
     ProjectIssueResourceStateEventManager,
+    ProjectIssueResourceWeightEventManager,
 )
 from .notes import ProjectIssueNoteManager  # noqa: F401
 
@@ -42,7 +45,7 @@ __all__ = [
 
 class Issue(RESTObject):
     _url = "/issues"
-    _short_print_attr = "title"
+    _repr_attr = "title"
 
 
 class IssueManager(RetrieveMixin, RESTManager):
@@ -108,7 +111,7 @@ class ProjectIssue(
     ObjectDeleteMixin,
     RESTObject,
 ):
-    _short_print_attr = "title"
+    _repr_attr = "title"
     _id_attr = "iid"
 
     awardemojis: ProjectIssueAwardEmojiManager
@@ -118,6 +121,8 @@ class ProjectIssue(
     resourcelabelevents: ProjectIssueResourceLabelEventManager
     resourcemilestoneevents: ProjectIssueResourceMilestoneEventManager
     resourcestateevents: ProjectIssueResourceStateEventManager
+    resource_iteration_events: ProjectIssueResourceIterationEventManager
+    resource_weight_events: ProjectIssueResourceWeightEventManager
 
     @cli.register_custom_action("ProjectIssue", ("to_project_id",))
     @exc.on_http_error(exc.GitlabUpdateError)
@@ -139,6 +144,38 @@ class ProjectIssue(
             assert isinstance(server_data, dict)
         self._update_attrs(server_data)
 
+    @cli.register_custom_action("ProjectIssue", ("move_after_id", "move_before_id"))
+    @exc.on_http_error(exc.GitlabUpdateError)
+    def reorder(
+        self,
+        move_after_id: Optional[int] = None,
+        move_before_id: Optional[int] = None,
+        **kwargs: Any,
+    ) -> None:
+        """Reorder an issue on a board.
+
+        Args:
+            move_after_id: ID of an issue that should be placed after this issue
+            move_before_id: ID of an issue that should be placed before this issue
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Raises:
+            GitlabAuthenticationError: If authentication is not correct
+            GitlabUpdateError: If the issue could not be reordered
+        """
+        path = f"{self.manager.path}/{self.encoded_id}/reorder"
+        data: Dict[str, Any] = {}
+
+        if move_after_id is not None:
+            data["move_after_id"] = move_after_id
+        if move_before_id is not None:
+            data["move_before_id"] = move_before_id
+
+        server_data = self.manager.gitlab.http_put(path, post_data=data, **kwargs)
+        if TYPE_CHECKING:
+            assert isinstance(server_data, dict)
+        self._update_attrs(server_data)
+
     @cli.register_custom_action("ProjectIssue")
     @exc.on_http_error(exc.GitlabGetError)
     def related_merge_requests(self, **kwargs: Any) -> Dict[str, Any]:
@@ -271,7 +308,7 @@ class ProjectIssueLinkManager(ListMixin, CreateMixin, DeleteMixin, RESTManager):
             GitlabAuthenticationError: If authentication is not correct
             GitlabCreateError: If the server cannot perform the request
         """
-        self._check_missing_create_attrs(data)
+        self._create_attrs.validate_attrs(data=data)
         if TYPE_CHECKING:
             assert self.path is not None
         server_data = self.gitlab.http_post(self.path, post_data=data, **kwargs)
